@@ -62,6 +62,10 @@ class gp_ec2_create_ami(Command):
         self.optparser.add_option("-t", "--instance-type", 
                                   action="store", type="string", dest="instance_type", 
                                   help = "EC2 instance type to use")
+
+        self.optparser.add_option("-g", "--security-groups", 
+                                  action="store", type="string", dest="security_groups", 
+                                  help = "EC2 security groups to use (comma separated)")
                 
     def run(self):    
         self.parse_options()
@@ -69,7 +73,12 @@ class gp_ec2_create_ami(Command):
         config = GPConfig(os.path.expanduser(self.opt.conf))
         chef_dir = os.path.expanduser(self.opt.chef_dir)
 
-        c = EC2AMICreator(chef_dir, self.opt.ami, self.opt.aminame, self.opt.instance_type, config)
+        if self.opt.security_groups == None:
+            sg = None
+        else:
+            sg = self.opt.security_groups.split(",")
+
+        c = EC2AMICreator(chef_dir, self.opt.ami, self.opt.aminame, self.opt.instance_type, sg, config)
         c.run()
         
         
