@@ -48,7 +48,7 @@ class GlobusOnlineHelper(object):
 
         gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject = self._get_hostnames_subjects(ep)
             
-        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, ep.public)       
+        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject, ep.public)       
 
     def _get_hostnames_subjects(self, ep):
         ca_dn = self.inst.config.get("ca-dn")
@@ -79,7 +79,14 @@ class GlobusOnlineHelper(object):
         if ep.myproxy.startswith("node:"):
             myproxy_node = self.inst.topology.get_node_by_id(ep.myproxy[5:])
             myproxy_hostname = myproxy_node.hostname
-            myproxy_subject = "%s/CN=host/%s" % (ca_dn, gridftp_hostname)
+            
+            if ep.has_property("globus_connect_cert") and ep.globus_connect_cert:
+                if ep.has_property("globus_connect_cert_dn"):
+                    myproxy_subject = ep.globus_connect_cert_dn
+                else:
+                    myproxy_subject = None
+            else:            
+                myproxy_subject = "%s/CN=host/%s" % (ca_dn, myproxy_hostname)
         else:
             myproxy_hostname = ep.myproxy  
             myproxy_subject = None
@@ -274,9 +281,12 @@ class GlobusOnlineCLIHelper(GlobusOnlineHelper):
         gridftp_subject = outf.getvalue().strip()      
         ep.set_property("globus_connect_cert_dn", gridftp_subject)
         
+        if ep.myproxy.startswith("node:"):
+            myproxy_subject = gridftp_subject
+        
         self.endpoint_remove(ep)
            
-        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, ep.public)
+        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject, ep.public)
 
     def endpoint_stop(self, ep):
         gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject = self._get_hostnames_subjects(ep)
@@ -284,7 +294,7 @@ class GlobusOnlineCLIHelper(GlobusOnlineHelper):
         self.endpoint_remove_server(ep, gridftp_hostname)        
         
         # Create a disconnected endpoint
-        self._endpoint_create(ep.name, "relay-disconnected.globusonline.org", gridftp_subject, "myproxy.globusonline.org", ep.public)
+        self._endpoint_create(ep.name, "relay-disconnected.globusonline.org", gridftp_subject, "myproxy.globusonline.org", None, ep.public)
 
     def endpoint_resume(self, ep):
         gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject = self._get_hostnames_subjects(ep)
@@ -292,9 +302,9 @@ class GlobusOnlineCLIHelper(GlobusOnlineHelper):
         # Remove disconnected server
         self.endpoint_remove_server(ep, "relay-disconnected.globusonline.org") 
         
-        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, ep.public)
+        self._endpoint_create(ep.name, gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject, ep.public)
 
-    def _endpoint_create(self, ep_name, gridftp_hostname, gridftp_subject, myproxy_hostname, public):   
+    def _endpoint_create(self, ep_name, gridftp_hostname, gridftp_subject, myproxy_hostname, myproxy_subject, public):   
         rc = self.ssh.run("endpoint-add %s -p %s -s \"%s\"" % (ep_name, gridftp_hostname, gridftp_subject), exception_on_error=False)
         if rc != 0:
             raise GlobusOnlineException, "Could not create endpoint %s" % ep_name
@@ -303,6 +313,11 @@ class GlobusOnlineCLIHelper(GlobusOnlineHelper):
         if rc != 0:
             raise GlobusOnlineException, "Could not set MyProxy server for endpoint %s" % ep_name
 
+        if myproxy_subject != None:
+            rc = self.ssh.run("endpoint-modify --myproxy-dn=\"%s\" %s" % (myproxy_subject, ep_name), exception_on_error=False)
+            if rc != 0:
+                raise GlobusOnlineException, "Could not set MyProxy subject for endpoint %s" % ep_name
+            
         if public:
             rc = self.ssh.run("endpoint-modify --public %s" % (ep_name), exception_on_error=False)
             if rc != 0:
